/**
 * Copyright (c) Members of the EGEE Collaboration. 2006-2010.
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Authors:
 * 2009-
 *      Oscar Koeroo <okoeroo@nikhef.nl>
 *      Mischa Sall\'e <msalle@nikhef.nl>
 *      NIKHEF Amsterdam, the Netherlands
 *
 *  2006-2009
 *      Gerben Venekamp <venekamp@nikhef.nl>
 *      NIKHEF Amsterdam, the Netherlands
 *
 */

/*!
 *  \file glexec_account.h
 *
 *  Provides helper methods to retrieve account information using getpwnam_r
 *  etc. but using caching.
 */

#ifndef _GLEXEC_ACCOUNT_H
#define _GLEXEC_ACCOUNT_H
 
/* Need for uid_t gid_t */
#include <sys/types.h>

/************************************************************************/
/* FUNCTIONS								*/
/************************************************************************/

/*!
 * substitute for getpwuid() using getpwuid_r() and caching.
 * Use glexec_free_cached_passwds() to clean up
 */
struct passwd *glexec_getpwuid(uid_t uid);

/*!
 * substitute for getpwnam() using getpwnam_r() and caching.
 * Use glexec_free_cached_passwds() to clean up
 */
struct passwd *glexec_getpwnam(const char *name);

/*!
 * substitute for getgrgid() using getgrgid() and caching
 * Use glexec_free_cached_groups() to clean up
 */
struct group *glexec_getgrgid(gid_t gid);

/*!
 * substitute for getgrnam() using getgrnam() and caching.
 * Use glexec_free_cached_groups() to clean up
 */
struct group *glexec_getgrnam(const char *name);

/*!
 * Free cached passwords
 */
void glexec_free_cached_passwds(void);

/*!
 * Free cached groups
 */ 
void glexec_free_cached_groups(void);

#endif
