/**
 * Copyright (c) Members of the EGEE Collaboration. 2010.
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Authors: Oscar Koeroo, Mischa Sall\'e, Aram Verstegen
 *      NIKHEF Amsterdam, the Netherlands
 *      <grid-mw-security@nikhef.nl>
 */

#ifndef _REALPATH_H_
#define _REALPATH_H_

#include <sys/stat.h>	/* Needed for definition of struct stat */

/*!
 * Returns the fully canonicalized and absolute path for a diretory or NULL on
 * error. Memory for the return string is malloced and should be freed by the
 * caller. Any path cannot be longer than PATH_MAX. Note that it is necessary
 * that we can chdir to dir.
 * NOTE: This function is not thread-safe
 * \param dir non-absolute and/or non-canonical directory
 * \return canonical absolute directory
 */
char *cgul_realdir(const char *dir);

/*!
 * Returns the fully canonicalized and absolute path for any path or NULL on
 * error. Note that this function substitutes the unsafe realpath() or the
 * GNU-only canonicalize_file_name().
 * Memory for the return string is malloced and should be freed by the
 * caller. Any path cannot be longer than PATH_MAX. Any directory has to be
 * accessible, including if it's the last element.
 * NOTE: This function is not thread-safe
 * \param inpath non-absolute and/or non-canonical path
 * \param instat optional stat information on the input (for performance, if
 *               already available). When NULL it will be determined locally
 *               when needed.
 * \return canonical absolute path
 */
char *cgul_realpath(const char *inpath, struct stat *instat);

#endif
