#!/bin/sh

#
# Copyright (c) Members of the EGEE Collaboration. 2009-2010.
# See http://www.eu-egee.org/partners/ for details on the copyright
# holders.
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# Authors:
# 2009-
#      Oscar Koeroo <okoeroo@nikhef.nl>
#      Mischa Sall\'e <msalle@nikhef.nl>
#      NIKHEF Amsterdam, the Netherlands
#

#################
# Setup options #
#################
CONTINUEONERROR=no
#CONTINUEONERROR=yes

TEST_ACCOUNT="okoeroo"

GLEXEC_EXEC="/opt/glite/sbin/glexec"
GLEXEC_OWNERSHIP_SETUID="root.root"
GLEXEC_FILE_PERM_SETUID="6555"
GLEXEC_OWNERSHIP_NON_SETUID="root.root"
GLEXEC_FILE_PERM_NON_SETUID="0555"

CONF_OWNERSHIP_SETUID="glexec.glexec"
CONF_FILE_PERM_SETUID="0440"
CONF_OWNERSHIP_NON_SETUID="root.root"
CONF_FILE_PERM_NON_SETUID="0444"

test_glexec_conf="/opt/glite/etc/glexec.conf"
test_lcas_db="/opt/glite/etc/lcas/lcas-testing.db"
test_lcas_db_path="/opt/glite/lib64/modules/"
test_lcmaps_db="/opt/glite/etc/lcmaps/lcmaps-testing.db"
test_lcmaps_db_path="/opt/glite/lib64/modules/"

priv_sep_file="/tmp/glexec_priv_sep_test.sh"
CAPATH="/etc/grid-security/certificates"
#SCAS_ENDPOINT="https://eir.nikhef.nl:8443"
SCAS_ENDPOINT="https://graszaad.nikhef.nl:8443"
PEPD_ENDPOINT="https://argus.cr.cnaf.infn.it:8154/authz"
GLEXEC_TEST_GRID_MAPFILE="/tmp/glexec-test-grid-mapfile"

LOCALACCOUNT_TEST_MAP_USER="$TEST_ACCOUNT"
#LOCALACCOUNT_TEST_MAP_USER="scas017"
LOCALACCOUNT_TEST_MAP_USER="pool005"


#################
# Setup proxies #
#################
CLIENT_CERT="/var/local/home/okoeroo/mkproxy-x509-voms"
USER_PROXY="$CLIENT_CERT"
SOURCE_PROXY="$CLIENT_CERT"
TARGET_PROXY="/tmp/target_proxy"

SUBJECT_DN=


#####################################################################
#####################################################################


function grab_subject_dn_from_proxy_cert () {
    SUBJECT_DN=`openssl x509 -subject -noout -in ${CLIENT_CERT} | sed -e 's/subject=\ //' -e 's/\/CN=proxy//g'`
}

# Read and set the $SUBJECT_DN from the $CLIENT_CERT
grab_subject_dn_from_proxy_cert
echo Subject DN to use: $SUBJECT_DN

if [ "$SUBJECT_DN" == "" ]; then
    echo No Subject DN registered
    exit 1
fi

function create_glexec_conf_file () {

cat > $test_glexec_conf <<End-of-message
#
#  Glexec
#
[glexec]
log_destination              = $LOG_DESTINATION
log_level                    = $LOG_LEVEL
log_file                     = $LOG_FILE

linger                       = $LINGER
silent_logging               = $SILENT_LOGGING
user_white_list              = $USER_WHITE_LIST

preserve_env_variables       = $PRESERVE_ENV_VARIABLES
user_identity_switch_by      = $USER_IDENTITY_SWITCH_BY

prohibit_exec_via_symlink    = $PROHIBIT_EXEC_VIA_SYMLINK
pedantic_security_checks     = $PEDANTIC_SECURITY_CHECKS

input_lock_mechanism         = $INPUT_LOCK_MECHANISM
target_lock_mechanism        = $TARGET_LOCK_MECHANISM

#use_lcas                     = $USE_LCAS

lcas_db_file                 = $LCAS_DB_FILE
lcas_log_file                = $LCAS_LOG_FILE
lcas_debug_level             = $LCAS_DEBUG_LEVEL
lcas_log_level               = $LCAS_LOG_LEVEL

lcmaps_db_file               = $LCMAPS_DB_FILE
lcmaps_get_account_policy    = $LCMAPS_GET_ACCOUNT_POLICY
lcmaps_log_file              = $LCMAPS_LOG_FILE
lcmaps_debug_level           = $LCMAPS_DEBUG_LEVEL
lcmaps_log_level             = $LCMAPS_LOG_LEVEL

End-of-message

chown $1 $3
chmod $2 $3
}


function create_lcas_db_file () {

cat > $test_lcas_db <<End-of-message
# LCAS policy file/plugin definition
# Written by: Oscar Koeroo - okoeroo * at * nikhef * dot * nl
pluginname=${test_lcas_db_path}lcas_userban.mod,pluginargs=ban_users.db
End-of-message
}

function create_lcmaps_db_file () {

# Write a dummy grid-mapfile
cat > $GLEXEC_TEST_GRID_MAPFILE <<End-of-message
"$SUBJECT_DN" $LOCALACCOUNT_TEST_MAP_USER
End-of-message

# Write the lcmaps.db file for testing, filled with various different scenarios
cat > $test_lcmaps_db <<End-of-message
# LCMAPS policy file/plugin definition
# Written by: Oscar Koeroo - okoeroo * at * nikhef * dot * nl
# The configuration file is specialized for non-root privileged processes/services, like:
### The WMProxy
### SCAS service
### gLExec in logging only mode (when it supports multiple evaluation policies)


# default path for the modules
path = $test_lcmaps_db_path

# Plugin definitions:
#good             = "lcmaps_dummy_good.mod"

good             = "lcmaps_dummy_good.mod"
                   " --dummy-username nobody"
                   " --dummy-group nobody"
                   " --dummy-sec-group nobody"

posix_enf        = "lcmaps_posix_enf.mod"
                   " -maxuid 1"
                   " -maxpgid 1"
                   " -maxsgid 32"

localaccount     = "lcmaps_localaccount.mod"
                   "-gridmapfile $GLEXEC_TEST_GRID_MAPFILE"

scasclient = "lcmaps_scas_client.mod"
             "-capath $CAPATH"
#             " -cert   /etc/grid-security/hostcert.pem"
#             " -key    /etc/grid-security/hostkey.pem"
             "--endpoint $SCAS_ENDPOINT"
#             "--retry 5"
#             "--endpoint-strategy round-robin"
#             "--endpoint-strategy round-robin-random-start"
#             "--endpoint-strategy random"
             "-resourcetype wn"
             "-actiontype execute-now"

verifyproxy = "lcmaps_verify_proxy.mod"
             " -certdir $CAPATH"
#             "--discard_private_key_absence"
#             "--never_discard_private_key_absence"

pepc        = "lcmaps_c_pep.mod"
              "--pep-daemon-endpoint-url $PEPD_ENDPOINT"
              "--resourceid http://cnaf.infn.it/wn"
              "--actionid http://glite.org/xacml/action/execute"
              "--capath /etc/grid-security/certificates"
              "--pep-certificate-mode implicit"

# Policies:
# SCAS-client

#verifyproxy ->  pepc
#pepc -> posix_enf

authzwg_posix:
verifyproxy -> pepc
pepc -> posix_enf 

scas_posix:
verifyproxy -> scasclient
scasclient -> posix_enf

authzwg_noposix:
verifyproxy -> pepc

scas_noposix:
verifyproxy -> scasclient

localmap_posix:
verifyproxy -> localaccount
localaccount -> posix_enf

localmap_noposix:
verifyproxy -> localaccount

End-of-message
}


function setup_glexec_conf () {
	LOG_DESTINATION="file"
	LOG_LEVEL="1"
	LOG_FILE="/var/log/glexec/myglexec.log"

	LINGER="yes"
	SILENT_LOGGING="no"
	USER_WHITE_LIST="$TEST_ACCOUNT"

	PRESERVE_ENV_VARIABLES="X509_USER_CERT, X509_USER_KEY"
	USER_IDENTITY_SWITCH_BY="lcmaps"

	PROHIBIT_EXEC_VIA_SYMLINK="no"
	PEDANTIC_SECURITY_CHECKS="no"

	INPUT_LOCK_MECHANISM="fcntl"
	TARGET_LOCK_MECHANISM="fcntl"

	#USE_LCAS

	LCAS_DB_FILE="/opt/glite/etc/lcas/lcas-testing.db"
	LCAS_LOG_FILE="/var/log/glexec/lcas_lcmaps.log"
	LCAS_LOG_LEVEL="1"
	LCAS_DEBUG_LEVEL="0"

	LCMAPS_DB_FILE="/opt/glite/etc/lcmaps/lcmaps-testing.db"
	LCMAPS_LOG_FILE="/var/log/glexec/lcas_lcmaps.log"
	LCMAPS_LOG_LEVEL="1"
	LCMAPS_DEBUG_LEVEL="0"
	LCMAPS_GET_ACCOUNT_POLICY="scas_posix"
}


function write_cmd_to_file () {
cat > $priv_sep_file <<End-of-message
#!/bin/sh

export X509_USER_PROXY=$USER_PROXY

export GLEXEC_CLIENT_CERT=$CLIENT_CERT
export GLEXEC_SOURCE_PROXY=$SOURCE_PROXY
export GLEXEC_TARGET_PROXY=$TARGET_PROXY

cd \$HOME

echo    "                          Executing: $1"
# echo -n "                          "
$1

exit \$?

End-of-message

chmod +x $priv_sep_file
}

#####################################################################

#The deployment scenarios:
#1.  setuid     / SCAS     / file logging / linger
#2.  setuid     / SCAS     / syslog       / linger
#3.  non-setuid / SCAS     / syslog       / linger
#4.  setuid     / non-SCAS / file logging / linger
#5.  setuid     / non-SCAS / syslog       / linger
#6.  non-setuid / non-SCAS / syslog       / linger
#7.  setuid     / SCAS     / file logging / don't linger
#8.  setuid     / SCAS     / syslog       / don't linger
#9.  non-setuid / SCAS     / syslog       / don't linger
#10. setuid     / non-SCAS / file logging / don't linger
#11. setuid     / non-SCAS / syslog       / don't linger
#12. non-setuid / non-SCAS / syslog       / don't linger


function setup_glexec_deployment_scene_1 () {
    echo "Deployment scenario  1 :: setting up: setuid, SCAS, file logging, linger"

    # Setup default setting for glexec.conf
    # Don't use as it will override ALL settings!
    # setup_glexec_conf

    # Special settings
    LINGER="yes"
    LOG_DESTINATION="file"
    LCMAPS_GET_ACCOUNT_POLICY="scas_posix"

    # Write the glexec.conf file
    create_glexec_conf_file $CONF_OWNERSHIP_SETUID $CONF_FILE_PERM_SETUID $test_glexec_conf

    # Set file permissions on the gLExec executable
    chown $GLEXEC_OWNERSHIP_SETUID $GLEXEC_EXEC
    chmod $GLEXEC_FILE_PERM_SETUID $GLEXEC_EXEC
}

function setup_glexec_deployment_scene_2 () {
    echo "Deployment scenario  2 :: setting up: setuid, SCAS, syslog, linger"

    # Setup default setting for glexec.conf
    # Don't use as it will override ALL settings!
    # setup_glexec_conf

    # Special settings
    LINGER="yes"
    LOG_DESTINATION="syslog"
    LCMAPS_GET_ACCOUNT_POLICY="scas_posix"

    # Write the glexec.conf file
    create_glexec_conf_file $CONF_OWNERSHIP_SETUID $CONF_FILE_PERM_SETUID $test_glexec_conf

    # Set file permissions on the gLExec executable
    chown $GLEXEC_OWNERSHIP_SETUID $GLEXEC_EXEC
    chmod $GLEXEC_FILE_PERM_SETUID $GLEXEC_EXEC
}

function setup_glexec_deployment_scene_3 () {
    echo "Deployment scenario  3 :: setting up: non-setuid, SCAS, syslog, linger"

    # Setup default setting for glexec.conf
    # Don't use as it will override ALL settings!
    # setup_glexec_conf

    # Special settings
    LINGER="yes"
    LOG_DESTINATION="syslog"
    LCMAPS_GET_ACCOUNT_POLICY="scas_noposix"

    # Write the glexec.conf file
    create_glexec_conf_file $CONF_OWNERSHIP_NON_SETUID $CONF_FILE_PERM_NON_SETUID $test_glexec_conf

    # Set file permissions on the gLExec executable
    chown $GLEXEC_OWNERSHIP_NON_SETUID $GLEXEC_EXEC
    chmod $GLEXEC_FILE_PERM_NON_SETUID $GLEXEC_EXEC
}

function setup_glexec_deployment_scene_4 () {
    echo "Deployment scenario  4 :: setting up: setuid, non-SCAS, file logging, linger"

    # Setup default setting for glexec.conf
    # Don't use as it will override ALL settings!
    # setup_glexec_conf

    # Special settings
    LINGER="yes"
    LOG_DESTINATION="file"
    LCMAPS_GET_ACCOUNT_POLICY="localmap_posix"

    # Write the glexec.conf file
    create_glexec_conf_file $CONF_OWNERSHIP_SETUID $CONF_FILE_PERM_SETUID $test_glexec_conf

    # Set file permissions on the gLExec executable
    chown $GLEXEC_OWNERSHIP_SETUID $GLEXEC_EXEC
    chmod $GLEXEC_FILE_PERM_SETUID $GLEXEC_EXEC
}

function setup_glexec_deployment_scene_5 () {
    echo "Deployment scenario  5 :: setting up: setuid, non-SCAS, syslog, linger"

    # Setup default setting for glexec.conf
    # Don't use as it will override ALL settings!
    # setup_glexec_conf

    # Special settings
    LINGER="yes"
    LOG_DESTINATION="syslog"
    LCMAPS_GET_ACCOUNT_POLICY="localmap_posix"

    # Write the glexec.conf file
    create_glexec_conf_file $CONF_OWNERSHIP_SETUID $CONF_FILE_PERM_SETUID $test_glexec_conf

    # Set file permissions on the gLExec executable
    chown $GLEXEC_OWNERSHIP_SETUID $GLEXEC_EXEC
    chmod $GLEXEC_FILE_PERM_SETUID $GLEXEC_EXEC
}

function setup_glexec_deployment_scene_6 () {
    echo "Deployment scenario  6 :: setting up: non-setuid, non-SCAS, syslog, linger"

    # Setup default setting for glexec.conf
    # Don't use as it will override ALL settings!
    # setup_glexec_conf

    # Special settings
    LINGER="yes"
    LOG_DESTINATION="syslog"
    LCMAPS_GET_ACCOUNT_POLICY="localmap_noposix"

    # Write the glexec.conf file
    create_glexec_conf_file $CONF_OWNERSHIP_NON_SETUID $CONF_FILE_PERM_NON_SETUID $test_glexec_conf

    # Set file permissions on the gLExec executable
    chown $GLEXEC_OWNERSHIP_NON_SETUID $GLEXEC_EXEC
    chmod $GLEXEC_FILE_PERM_NON_SETUID $GLEXEC_EXEC
}

function setup_glexec_deployment_scene_7 () {
    echo "Deployment scenario  7 :: setting up: setuid, SCAS, file logging, don't linger"

    # Setup default setting for glexec.conf
    # Don't use as it will override ALL settings!
    # setup_glexec_conf

    # Special settings
    LINGER="no"
    LOG_DESTINATION="file"
    LCMAPS_GET_ACCOUNT_POLICY="scas_posix"

    # Write the glexec.conf file
    create_glexec_conf_file $CONF_OWNERSHIP_SETUID $CONF_FILE_PERM_SETUID $test_glexec_conf

    # Set file permissions on the gLExec executable
    chown $GLEXEC_OWNERSHIP_SETUID $GLEXEC_EXEC
    chmod $GLEXEC_FILE_PERM_SETUID $GLEXEC_EXEC
}

function setup_glexec_deployment_scene_8 () {
    echo "Deployment scenario  8 :: setting up: setuid, SCAS, syslog, don't linger"

    # Setup default setting for glexec.conf
    # Don't use as it will override ALL settings!
    # setup_glexec_conf

    # Special settings
    LINGER="no"
    LOG_DESTINATION="syslog"
    LCMAPS_GET_ACCOUNT_POLICY="scas_posix"

    # Write the glexec.conf file
    create_glexec_conf_file $CONF_OWNERSHIP_SETUID $CONF_FILE_PERM_SETUID $test_glexec_conf

    # Set file permissions on the gLExec executable
    chown $GLEXEC_OWNERSHIP_SETUID $GLEXEC_EXEC
    chmod $GLEXEC_FILE_PERM_SETUID $GLEXEC_EXEC
}

function setup_glexec_deployment_scene_9 () {
    echo "Deployment scenario  9 :: setting up: non-setuid, SCAS, syslog, don't linger"

    # Setup default setting for glexec.conf
    # Don't use as it will override ALL settings!
    # setup_glexec_conf

    # Special settings
    LINGER="no"
    LOG_DESTINATION="syslog"
    LCMAPS_GET_ACCOUNT_POLICY="scas_noposix"

    # Write the glexec.conf file
    create_glexec_conf_file $CONF_OWNERSHIP_NON_SETUID $CONF_FILE_PERM_NON_SETUID $test_glexec_conf

    # Set file permissions on the gLExec executable
    chown $GLEXEC_OWNERSHIP_NON_SETUID $GLEXEC_EXEC
    chmod $GLEXEC_FILE_PERM_NON_SETUID $GLEXEC_EXEC
}

function setup_glexec_deployment_scene_10() {
    echo "Deployment scenario 10 :: setting up: setuid, non-SCAS, file logging, don't linger"

    # Setup default setting for glexec.conf
    # Don't use as it will override ALL settings!
    # setup_glexec_conf

    # Special settings
    LINGER="no"
    LOG_DESTINATION="file"
    LCMAPS_GET_ACCOUNT_POLICY="localmap_posix"

    # Write the glexec.conf file
    create_glexec_conf_file $CONF_OWNERSHIP_SETUID $CONF_FILE_PERM_SETUID $test_glexec_conf

    # Set file permissions on the gLExec executable
    chown $GLEXEC_OWNERSHIP_SETUID $GLEXEC_EXEC
    chmod $GLEXEC_FILE_PERM_SETUID $GLEXEC_EXEC
}

function setup_glexec_deployment_scene_11 () {
    echo "Deployment scenario 11 :: setting up: setuid, non-SCAS, syslog, don't linger"

    # Setup default setting for glexec.conf
    # Don't use as it will override ALL settings!
    # setup_glexec_conf

    # Special settings
    LINGER="no"
    LOG_DESTINATION="syslog"
    LCMAPS_GET_ACCOUNT_POLICY="localmap_posix"

    # Write the glexec.conf file
    create_glexec_conf_file $CONF_OWNERSHIP_SETUID $CONF_FILE_PERM_SETUID $test_glexec_conf

    # Set file permissions on the gLExec executable
    chown $GLEXEC_OWNERSHIP_SETUID $GLEXEC_EXEC
    chmod $GLEXEC_FILE_PERM_SETUID $GLEXEC_EXEC
}

function setup_glexec_deployment_scene_12 () {
    echo "Deployment scenario 12 :: setting up: non-setuid, non-SCAS, syslog, don't linger"

    # Setup default setting for glexec.conf
    # Don't use as it will override ALL settings!
    # setup_glexec_conf

    # Special settings
    LINGER="no"
    LOG_DESTINATION="syslog"
    LCMAPS_GET_ACCOUNT_POLICY="localmap_noposix"

    # Write the glexec.conf file
    create_glexec_conf_file $CONF_OWNERSHIP_NON_SETUID $CONF_FILE_PERM_NON_SETUID $test_glexec_conf

    # Set file permissions on the gLExec executable
    chown $GLEXEC_OWNERSHIP_NON_SETUID $GLEXEC_EXEC
    chmod $GLEXEC_FILE_PERM_NON_SETUID $GLEXEC_EXEC
}


#####################################################################

function test_result_expectation_handler () {
    if [ $1 -ne $2 ]; then
        echo
        echo -n "--- Test failure detected! Test returned $1, the exepected result was $2 ---"
        if [ ! "$CONTINUEONERROR" == "yes" ]; then
            echo
            echo 
            exit 1;
        else
            echo " continueing..."
            echo 
        fi
    fi
}

function execute_cmd_from_test_account () {

    EXPECTED_RESULT=$1

    # Shift input variables to seperate the commands and arguments from the $1 value
    shift
    
    # Write the command into a script. This script will be executed as the $TEST_ACCOUNT from $TEST_ACCOUNT's environment
    write_cmd_to_file "$*"

    echo "Executing deployment scenarios :: su $TEST_ACCOUNT --command=$priv_sep_file :: \"$*\""

    ### Deployment scenario 1 ###
    setup_glexec_deployment_scene_1
    rm -f $TARGET_PROXY

    su $TEST_ACCOUNT --command=$priv_sep_file
    rc=$?

    # Grab the result and compare with the expected result
    echo "Deployment scenario  1 :: Result: $rc - Expected: $EXPECTED_RESULT"
    test_result_expectation_handler $rc $EXPECTED_RESULT


    ### Deployment scenario 2 ###
    setup_glexec_deployment_scene_2
    rm -f $TARGET_PROXY

    su $TEST_ACCOUNT --command=$priv_sep_file
    rc=$?

    # Grab the result and compare with the expected result
    echo "Deployment scenario  2 :: Result: $rc - Expected: $EXPECTED_RESULT"
    test_result_expectation_handler $rc $EXPECTED_RESULT

    
    ### Deployment scenario 3 ###
    setup_glexec_deployment_scene_3
    rm -f $TARGET_PROXY

    su $TEST_ACCOUNT --command=$priv_sep_file
    rc=$?

    # Grab the result and compare with the expected result
    echo "Deployment scenario  3 :: Result: $rc - Expected: $EXPECTED_RESULT"
    test_result_expectation_handler $rc $EXPECTED_RESULT

    ### Deployment scenario 4 ###
    setup_glexec_deployment_scene_4
    rm -f $TARGET_PROXY

    su $TEST_ACCOUNT --command=$priv_sep_file
    rc=$?

    # Grab the result and compare with the expected result
    echo "Deployment scenario  4 :: Result: $rc - Expected: $EXPECTED_RESULT"
    test_result_expectation_handler $rc $EXPECTED_RESULT

    ### Deployment scenario 5 ###
    setup_glexec_deployment_scene_5
    rm -f $TARGET_PROXY

    su $TEST_ACCOUNT --command=$priv_sep_file
    rc=$?

    # Grab the result and compare with the expected result
    echo "Deployment scenario  5 :: Result: $rc - Expected: $EXPECTED_RESULT"
    test_result_expectation_handler $rc $EXPECTED_RESULT

    ### Deployment scenario 6 ###
    setup_glexec_deployment_scene_6
    rm -f $TARGET_PROXY

    su $TEST_ACCOUNT --command=$priv_sep_file
    rc=$?

    # Grab the result and compare with the expected result
    echo "Deployment scenario  6 :: Result: $rc - Expected: $EXPECTED_RESULT"
    test_result_expectation_handler $rc $EXPECTED_RESULT
 
    ### Deployment scenario 7 ###
    setup_glexec_deployment_scene_7
    rm -f $TARGET_PROXY

    su $TEST_ACCOUNT --command=$priv_sep_file
    rc=$?

    # Grab the result and compare with the expected result
    echo "Deployment scenario  7 :: Result: $rc - Expected: $EXPECTED_RESULT"
    test_result_expectation_handler $rc $EXPECTED_RESULT

    ### Deployment scenario 8 ###
    setup_glexec_deployment_scene_8
    rm -f $TARGET_PROXY

    su $TEST_ACCOUNT --command=$priv_sep_file
    rc=$?

    # Grab the result and compare with the expected result
    echo "Deployment scenario  8 :: Result: $rc - Expected: $EXPECTED_RESULT"
    test_result_expectation_handler $rc $EXPECTED_RESULT

    ### Deployment scenario 9 ###
    setup_glexec_deployment_scene_9
    rm -f $TARGET_PROXY

    su $TEST_ACCOUNT --command=$priv_sep_file
    rc=$?

    # Grab the result and compare with the expected result
    echo "Deployment scenario  9 :: Result: $rc - Expected: $EXPECTED_RESULT"
    test_result_expectation_handler $rc $EXPECTED_RESULT

    ### Deployment scenario 10 ###
    setup_glexec_deployment_scene_10
    rm -f $TARGET_PROXY
  
    su $TEST_ACCOUNT --command=$priv_sep_file
    rc=$?
  
    # Grab the result and compare with the expected result
    echo "Deployment scenario 10 :: Result: $rc - Expected: $EXPECTED_RESULT"
    test_result_expectation_handler $rc $EXPECTED_RESULT
  
    ### Deployment scenario 11 ###
    setup_glexec_deployment_scene_11
    rm -f $TARGET_PROXY
  
    su $TEST_ACCOUNT --command=$priv_sep_file
    rc=$?
  
    # Grab the result and compare with the expected result
    echo "Deployment scenario 11 :: Result: $rc - Expected: $EXPECTED_RESULT"
    test_result_expectation_handler $rc $EXPECTED_RESULT
  
    ### Deployment scenario 12 ###
    setup_glexec_deployment_scene_12
    rm -f $TARGET_PROXY
  
    su $TEST_ACCOUNT --command=$priv_sep_file
    rc=$?
  
    # Grab the result and compare with the expected result
    echo "Deployment scenario 12 :: Result: $rc - Expected: $EXPECTED_RESULT"
    test_result_expectation_handler $rc $EXPECTED_RESULT


    return $rc
}

#####################################################################

function test_1 () {
    echo "Test 1: Basic functional gLExec test (all must be working)"

    # Restore settings to default before compound test is executed 
    setup_glexec_conf

    # execute test in deployment scenarios
    execute_cmd_from_test_account 0   $GLEXEC_EXEC /usr/bin/id -a
}

function test_2 () {
    echo "Test 2: gLExec executing a non-existing executable"

    # Restore settings to default before compound test is executed 
    setup_glexec_conf

    # execute test in deployment scenarios
    execute_cmd_from_test_account 201 $GLEXEC_EXEC I_do_not_exist
}

function test_3 () {
    echo "Test 3: gLExec try to execute a symlinked binary (success)"

    # Restore settings to default before compound test is executed 
    setup_glexec_conf
    PROHIBIT_EXEC_VIA_SYMLINK="no"

    # execute test in deployment scenarios
    execute_cmd_from_test_account 0   $GLEXEC_EXEC /bin/sh -c echo hello glexec world
}

function test_4 () {
    echo "Test 4: gLExec tries to execute a symlinked binary (prohibited/protected)"

    # Restore settings to default before compound test is executed 
    setup_glexec_conf
    PROHIBIT_EXEC_VIA_SYMLINK="yes"

    # execute test in deployment scenarios
    execute_cmd_from_test_account 201 $GLEXEC_EXEC /bin/sh -c echo hello glexec world
}

function test_5_1 () {
    echo "Test 5_1: gLExec tries to execute a file that isn't executable"

    # Restore settings to default before compound test is executed 
    setup_glexec_conf

    TEST_FILE1="/tmp/glexec_test_file_to_test_with.sh"
    > $TEST_FILE1
    chmod a-x $TEST_FILE1

    # execute test in deployment scenarios
    execute_cmd_from_test_account 126 $GLEXEC_EXEC $TEST_FILE1

    rm $TEST_FILE1
}

function test_5_2 () {
    echo "Test 5_2: gLExec tries to execute a file that isn't a file"

    # Restore settings to default before compound test is executed 
    setup_glexec_conf

    TEST_FILE1="/tmp/glexec_test_file_to_test_with.sh"
    mkfifo $TEST_FILE1

    # execute test in deployment scenarios
    execute_cmd_from_test_account 201 $GLEXEC_EXEC $TEST_FILE1

    rm $TEST_FILE1
}

function test_6 () {
    echo "Test 6: gLExec tries to execute a file that isn't executable, but now the pedantic checks are enabled, still same return value"

    # Restore settings to default before compound test is executed 
    setup_glexec_conf
    PEDANTIC_SECURITY_CHECKS="yes"

    TEST_FILE1="/tmp/glexec_test_file_to_test_with.sh"
    > $TEST_FILE1
    chmod a-x $TEST_FILE1

    # execute test in deployment scenarios
    execute_cmd_from_test_account 126 $GLEXEC_EXEC $TEST_FILE1

    rm $TEST_FILE1
}

function test_7 () {
    echo "Test 7: gLExec tries to execute a file that is executable, but now unreachable by ownership and file permissions."

    # Restore settings to default before compound test is executed 
    setup_glexec_conf
    PEDANTIC_SECURITY_CHECKS="no"

    TEST_FILE1="/tmp/glexec_test_file_to_test_with.sh"
    > $TEST_FILE1
    chmod 0750 $TEST_FILE1
    chown root.root $TEST_FILE1

    # execute test in deployment scenarios
    execute_cmd_from_test_account 126 $GLEXEC_EXEC $TEST_FILE1

    rm $TEST_FILE1
}

function test_8 () {
    echo "Test 8: gLExec tries to execute a file that is executable, but now unreachable by ownership and file permissions, with pedenatic file checks on"

    # Restore settings to default before compound test is executed 
    setup_glexec_conf
    PEDANTIC_SECURITY_CHECKS="yes"

    TEST_FILE1="/tmp/glexec_test_file_to_test_with.sh"
    > $TEST_FILE1
    chmod 0750 $TEST_FILE1
    chown root.root $TEST_FILE1

    # execute test in deployment scenarios
    execute_cmd_from_test_account 126 $GLEXEC_EXEC $TEST_FILE1

    rm $TEST_FILE1
}

function test_9 () {
    echo "Test 9: gLExec testing lock mechansims: input locking: $1 for target locking: $2"

    # Restore settings to default before compound test is executed 
    setup_glexec_conf

    INPUT_LOCK_MECHANISM=$1
    TARGET_LOCK_MECHANISM=$2

    # execute test in deployment scenarios
    execute_cmd_from_test_account 0   $GLEXEC_EXEC /usr/bin/id -a
}

function test_10 () {
    echo "Test 10: gLExec tries to execute a setuid-enabled executable"

    # Restore settings to default before compound test is executed 
    setup_glexec_conf

    TEST_FILE1="/tmp/glexec_test_file_to_test_with.sh"
    cat > $TEST_FILE1 <<End-of-message
#!/bin/sh
End-of-message
    chmod 4555 $TEST_FILE1
    chown root.root $TEST_FILE1

    # execute test in deployment scenarios
    execute_cmd_from_test_account 0 $GLEXEC_EXEC $TEST_FILE1

    rm $TEST_FILE1
}

function test_11 () {
    echo "Test 11: gLExec tries to execute a setgid-enabled executable."

    # Restore settings to default before compound test is executed 
    setup_glexec_conf

    TEST_FILE1="/tmp/glexec_test_file_to_test_with.sh"
    rm $TEST_FILE1
    cat > $TEST_FILE1 <<End-of-message
#!/bin/sh
End-of-message
    chmod 2555 $TEST_FILE1
    chown root.root $TEST_FILE1

    # execute test in deployment scenarios
    execute_cmd_from_test_account 0 $GLEXEC_EXEC $TEST_FILE1

    rm $TEST_FILE1
}

function test_12 () {
    echo "Test 12: gLExec tries to execute a setuid-setgid-enabled executable."

    # Restore settings to default before compound test is executed 
    setup_glexec_conf

    TEST_FILE1="/tmp/glexec_test_file_to_test_with.sh"
    rm $TEST_FILE1
    cat > $TEST_FILE1 <<End-of-message
#!/bin/sh
End-of-message
    chmod 6555 $TEST_FILE1
    chown root.root $TEST_FILE1

    # execute test in deployment scenarios
    execute_cmd_from_test_account 0 $GLEXEC_EXEC $TEST_FILE1

    rm $TEST_FILE1
}

function test_13 () {
    echo "Test 13: gLExec tries to execute a world-writeable executable."

    # Restore settings to default before compound test is executed 
    setup_glexec_conf

    TEST_FILE1="/tmp/glexec_test_file_to_test_with.sh"
    cat > $TEST_FILE1 <<End-of-message
#!/bin/sh
End-of-message

    chmod 0777  $TEST_FILE1
    chown root.root $TEST_FILE1

    # execute test in deployment scenarios
    # execute_cmd_from_test_account 201 $GLEXEC_EXEC $TEST_FILE1
    execute_cmd_from_test_account 0 $GLEXEC_EXEC $TEST_FILE1

    rm $TEST_FILE1
}

function test_14 () {
    echo "Test 14: gLExec tries to execute a world-writeable executable, with pedantic checks are now enabled."

    # Restore settings to default before compound test is executed 
    setup_glexec_conf
    PEDANTIC_SECURITY_CHECKS="yes"

    TEST_FILE1="/tmp/glexec_test_file_to_test_with.sh"
    cat > $TEST_FILE1 <<End-of-message
#!/bin/sh
End-of-message

    chmod 0777  $TEST_FILE1
    chown root.root $TEST_FILE1

    # execute test in deployment scenarios
    execute_cmd_from_test_account 201 $GLEXEC_EXEC $TEST_FILE1

    rm $TEST_FILE1
}



function test_execution_of_all_tests_in_all_scenarios () {
    echo "Starting all tests..."

    echo "----- test_1 -----"
    test_1

    echo "----- test_2 -----"
    test_2

    echo "----- test_3 -----"
    test_3

    echo "----- test_4 -----"
    test_4

    echo "----- test_5_1 -----"
    test_5_1

    echo "----- test_5_2 -----"
    test_5_2

    echo "----- test_6 -----"
    test_6

    echo "----- test_7 -----"
    test_7

    echo "----- test_8 -----"
    test_8

    echo "----- test_9.1 -----"
    test_9 disabled disabled

    echo "----- test_9.2 -----"
    test_9 flock    disabled

    echo "----- test_9.3 -----"
    test_9 disabled flock

    echo "----- test_9.4 -----"
    test_9 flock    flock

    echo "----- test_9.5 -----"
    test_9 fcntl    disabled

    echo "----- test_9.6 -----"
    test_9 disabled fcntl 

    echo "----- test_9.7 -----"
    test_9 fcntl    flock

    echo "----- test_9.8 -----"
    test_9 flock    fcntl 

    echo "----- test_9.9 -----"
    test_9 fcntl    fcntl 

    echo "----- test_10 -----"
    test_10

    echo "----- test_11 -----"
    test_11

    echo "----- test_12 -----"
    test_12

    echo "----- test_13 -----"
    test_13

    echo "----- test_14 -----"
    test_14
}


#####################################################################
# Start of the program
#####################################################################

echo "This is the gLExec testing facility"
echo 
echo "----- Warning!!! --  Config files will be overwritten, only a few seconds left to stop now  --  Warning!!! -----"

#sleep 4

echo "Starting test sequence..."

echo -n "Wait a minute, are you root? "
if [ `id -u` -ne 0 ]; then
    echo Epic fail
    exit 1;
else
    echo Excellent.
fi

echo "Showing compile options and version:"
$GLEXEC_EXEC -v
$GLEXEC_EXEC -V

echo "Setup gLExec configuration file basic values"
setup_glexec_conf


echo "Setting up configuration for LCAS to '${test_lcas_db}'"
create_lcas_db_file
echo "Setting up configuration for LCMAPS to '${test_lcmaps_db}'"
create_lcmaps_db_file



if [ ! -e /opt/glite/etc/glexec.conf.saved ]; then
    echo "Saving glexec.conf to '/opt/glite/etc/glexec.conf.saved'"
    mv /opt/glite/etc/glexec.conf /opt/glite/etc/glexec.conf.saved
fi


#####################################################################
# Execute all tests
#####################################################################
test_execution_of_all_tests_in_all_scenarios


#####################################################################
# Finish
#####################################################################
echo 
echo 
if [ "$CONTINUEONERROR" == "yes" ]; then
    echo "Test sequence finished with UNKNOWN success, because \$CONTINUEONERROR=\"yes\""
else
    echo "Test sequence finished with SUCCESS."
fi

echo
echo "Returning back to deployment scenario 1 (setuid, SCAS, file-logging)"
setup_glexec_deployment_scene_1
echo 
echo done.
